use crate::smallint::SmallIntType;
use crate::SmallInt;
use crate::SmallUint;
use std::cmp::Ordering;

impl PartialOrd for SmallInt {
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        let a_sign;
        match &self.0 {
            SmallIntType::Inline(i) => a_sign = i.signum() as i8,
            SmallIntType::Heap((_, s)) => a_sign = s.signum() as i8,
        }

        let b_sign;
        match &other.0 {
            SmallIntType::Inline(i) => b_sign = i.signum() as i8,
            SmallIntType::Heap((_, s)) => b_sign = s.signum() as i8,
        }

        match (a_sign, b_sign) {
            x if (x.0 >= 0 && x.1 < 0) => Some(Ordering::Greater),

            x if (x.0 < 0 && x.1 >= 0) => Some(Ordering::Less),

            x if (x.0 >= 0 && x.1 >= 0) => SmallUint::from_smallint_unsigned(self.clone())
                .partial_cmp(&SmallUint::from_smallint_unsigned(other.clone())),

            x if (x.0 < 0 && x.1 < 0) => SmallUint::from_smallint_unsigned(other.clone())
                .partial_cmp(&SmallUint::from_smallint_unsigned(self.clone())),

            (_, _) => None,
        }
    }
}

impl Ord for SmallInt {
    fn cmp(&self, other: &Self) -> Ordering {
        self.partial_cmp(other).expect("This should not happen.")
    }
}
